//
//  main.cpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 22/06/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#include "/Users/bouaniche/nps/soot_stochastic/main.hpp"


#include <fstream>

using namespace std;

int main()
{
    
    // Paths
    string pathProject("./");
    
    // Output files
    string pathTargetPsd("outputs/psd/psd");
    string pathTargetGlobal("outputs/global/global");
    
    // Input files
    //string pathTagetInputsHomogeneousPhase("../burnerFlame/outputs/data_parsed/");
    string pathTagetInputsHomogeneousPhase("inputs/");
    //string HomogeneousPhaseDataFileName("data.txt");
    string HomogeneousPhaseDataFileName("R_data.txt");
    string HomogeneousPhaseLabelFileName("labels.txt");

    
    //user inputs NPS *******************************************************************
   
    // Problem solved
    string solverType = "HomogeneousGasPhase";
    //string solverType = "analytical";
    
    // PBE solving
    //string pbeMethod = "npBasic";
    //string pbeMethod = "npBasicPart";
    //string pbeMethod = "npResidual";
    //string pbeMethod = "npResidualPart";
    //string pbeMethod = "sectional";
    string pbeMethod = "hybrid";
    //string pbeMethod = "hybridPart";
    
    // sectional method type (for growth)
    string sectionalMethod = "park";
    //string sectionalMethod = "2pointUpwindmAvg";
    //string sectionalMethod = "2pointUpwindmSup";
    
    
    int partPerCluster = 1e2;
    //int partPerCluster = 1;  // !!!  set as 1 if clusters not used  !!!
    
    int hybridTrigger = 5; //triggers if 0 < npi < hybridTrigger
    //int hybridTrigger = 2;  // if set at 0, will never activate
    
    
    //Grid (particle mass) parameters
    // Type
    //string gridType = "standard";
    //string gridType = "geometric2";
    string gridType = "geometricFs";
    //string gridType = "regular";
    //string gridType = "exponential";
    
    // All Grids Parameters
    int nBins = 30;
    //int nBins = 25;
    
    // for "standard" grids
    
    /*
    double factorFirstCell = 2.0;
    double c2coef = 2*12*1.6605E-24;  // for "standard" gridType
    double mMin = 5.98E-22;
    double mSupMax = 5.0e-15;
    */
    
    double factorFirstCell = 0.0;
    double c2coef = 0.0;  // for "standard" gridType
    double mMin = 0.0;
    double mSupMax = 0.0;
    
     
    // cmass = 12.0 * 1.6605e-24;
    // mPrimaryPart = 2*16*cmass
    
    // for "geometricFs" grids or "exponential" grids
    
    // 1: for hybrid:
    // mMin * (1 + AllocNuclRatio)/2  (par ex pour alloc sur [mMin; mMin*1.1] avec AllocNuclRatio = 1.1,  should correspond to mPrimaryPart
    // so mMin = mPrimaryPart * 2 / (1 + allocNuclRatio)
    // then firstCellmAvg can be deduced with this relation double mInf0 = 2.0* firstCellmAvg / (1.0+fs);
    // then firstCellmAvg = mMin * (1+fs)/2
    
    // 2: for sectional, should have firstCellmAvg = mPrimaryPart?:
    //double allocNuclRatio = 1.3; // !! must be the same as cste in reallocate
    double mPrimaryPart = 2.0 * 16.0 * 12.0 * 1.6605e-24;
    double fs = 1.5;
    //double firstCellmAvg = 6.4e-22; // for "geometric2" or "geometricFs" gridType
    //double firstCellmAvg = mPrimaryPart * 2.0 / (1.0 + allocNuclRatio) * (1.0+fs)/2.0; // for "geometric2" or "geometricFs" gridType
    
    //double firstCellmAvg = mPrimaryPart; // for "geometric2" or "geometricFs" gridType
    double firstCellmAvg = mPrimaryPart * 1.25; // so than mInf0 = mPrimaryPart
    
    
    
    // for "regular" grids: geometricFs parameters + gridIncrement
    double gridIncrement = 0.0;
    
    // for "exponential" grids: fill in mMin in "standard" grid section or here
    double alphaGrid = 0.0;
    //double mMin = 2.0/3.0 * 0.1;
    
    string initType = "mass";
    //string initType = "diameter";
    
    // Initial AerosolPhase (Psd)
    //string initShape = "exponential";
    string initShape = "binInit";
    int npTot = 1.0e4;
    
    //exponential
    //double n0 = 1.0e-20;
    double n0 = 1.0;
    //double x0 = 0.5;
    double x0 = firstCellmAvg;
    
    // binInit
    //int binInit = 0;
    //double initNvTbin = 1.0/1.5; // initial NvT. not necessary if creator with n0 and x0 is used.
    int binInit = 0;
    double initNvTbin = 1.0;
    
    
    // Experimental: smallest particles detected for output
    double smallestDetectedDiameter = 3.0; // in [nm]
    //double smallestDetectedDiameter = 0.0; // in [nm]
    
    // SootSourceTerms model parameters
    
    // Type
    string aggloType = "standard";
    //string aggloType = "harmonicAvgDc";  // Dc calculated based on fractal law
    //string aggloType = "betaConst";
    
    string surfType = "sphere";
    //string surfType = "fractal";       // Surface for reaction calculated based on fractal law
    
    //string massTransferType = "aggloTensor";
    string massTransferType = "kumar96";
    //string massTransferType = "geo2SPAMM";  // only for "geometric2" gridType
    
    string nuclType = "kraft";
    //string nuclType = "exponential";  // only for analytical test case because nucleation at all sizes
    //string nuclType = "dotHconst";  //
    
    string PAHType = "corrected";
    string A4correctionType = "PAHsources";
    
    string growthType = "standard"; // size/chemistry dependant
    //string growthType = "G0mTheta";
    
    //string growthType = "constant";  // analytical
    //string growthType = "linear";  // analytical
    
    string HACAType = "appel";
    //string HACAType = "constant";
    
    
    
    // source constants
    //double beta0 = 0.0; // agglo
    //double g0 = 1.0;  // growth
    //double b0 = 1.0e0; // nucleation / exponential nucleation
    //double x0n = 0.0;  // exponential nucleation
    double alphaHaca = 1.0;
    
    // parameters for GrowthTensor
    //double teta = 1.0;
    //double Malpha = 1.0e-5;
    
    //aggregate shape model
    double primaryDiameter = 5.0; //imposed primary particle diameter in nanometers.
    double fractalDim = 1.73;  // from YON
    double kf = 1.94;  // from Yon
    
    // TimeStep settings
    
    //string tStepType = "constant";
    //string tStepType = "fixedAlpha";
    //string tStepType = "growth";
    string tStepType = "alphaAndgSubsteps";
    
    //string growthtStepType = "growthStochastic";
    string growthtStepType = "growthSectional";
    
    double alphaAgglo = 0.1;
    
    double maxGcoef = 0.1;
    
    // NpSolver settings
    
    // fixed time step
    //int itTotal = 300;
    double timePerIt = 1.0e-4;
    
    int itTotal = 1;
    //double timePerIt = 1.0e-20;
    
    // variable time step
    double tFinal = 3.0e-2;
    
    
    int writeFreq = 1; // NpSolver will write outputs every writeFreq iteration
    int showPsdFreq = 1e2; // NpSolver will make a console print every showPsdFreq iteration
    
    // writing additional writePsd, for growth subIteration within one agglo iteration. Just for a specified time range
    bool subItWrite = true;
    //bool subItWrite = false;
    double subItWrite_tmin = 0.9*tFinal;
    double subItWrite_tmax = tFinal;
    
    
    
    //end user inputs  *******************************************************************
    
    
    //Objects and pointers Initialization. Solving
    
    if(solverType == "HomogeneousGasPhase")
    {
        mainHomogeneousGasPhaseStandard(pathProject, pathTagetInputsHomogeneousPhase, HomogeneousPhaseDataFileName, HomogeneousPhaseLabelFileName, gridType, nBins, mSupMax, factorFirstCell, mMin, c2coef, n0, x0, npTot, initType, smallestDetectedDiameter, solverType, aggloType, kf, primaryDiameter, fractalDim, surfType, massTransferType, nuclType, PAHType, A4correctionType,growthType, HACAType, itTotal, timePerIt, pathTargetPsd, pathTargetGlobal, alphaHaca, tStepType, alphaAgglo, maxGcoef, sectionalMethod, fs, growthtStepType, tFinal, partPerCluster, writeFreq, hybridTrigger, showPsdFreq, subItWrite, subItWrite_tmin, subItWrite_tmax, pbeMethod, firstCellmAvg, gridIncrement, alphaGrid, initShape, initNvTbin, binInit);
    }
    else if(solverType == "analytical")
    {
        //mainAnalyticalStandard(gridType, nBins, mSupMax, factorFirstCell, mMin, n0, x0, npTot, initType, solverType, aggloType, beta0, massTransferType, nuclType, b0, x0n, growthType, g0, itTotal, timePerIt, pathProject, pathTargetPsd, pathTargetGlobal, c2coef, firstCellmAvg, fs, initShape, binInit, initNvTbin, smallestDetectedDiameter, pbeMethod, teta, Malpha, alphaAgglo, maxGcoef, sectionalMethod, tStepType, growthtStepType, partPerCluster, hybridTrigger, tFinal, writeFreq, showPsdFreq, subItWrite, subItWrite_tmin, subItWrite_tmax, gridIncrement, alphaGrid);
        
        
    }
    else
    {
        cout << "ERROR: solverType" << endl;
    }
    
    
    
    return 0;
}
