//
//  AerosolPhase.hpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 29/08/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#ifndef AerosolPhase_hpp
#define AerosolPhase_hpp

#include <stdio.h>
#include <vector>

#include "SootStochasticParticle.hpp"
#include "GasParticle.hpp"
#include "Grid.hpp"
#include "GridCell.hpp"

#include "streams.hpp"
#include "tools.hpp"

class AerosolPhase
{
public:
    AerosolPhase();
    AerosolPhase(Grid* gridPtr,int npTot); // all particles are randomly allocated in first bin
    AerosolPhase(Grid* gridPtr, double n0, double x0, int npTot, std::string initType); // distribution in diameter or mass. // initType: "diameter", "mass"
        //initial exponential distribution in the form nd(dAvg) = n0/x0 * exp(-dAvg/x0). !! nd != nm and dAvg != mAvg !!  or nd(m) = n0/x0 * exp(-m/x0) depending on initType. in diameter nd = dNv / dlog(d) = Nv / (log(dsup) - log(dinf) in mass ndm = dNv / dm = Nv / (mSup - minf)
    
    //  !!!  dAvg in [nanometers]  !!
    
    AerosolPhase(std::string pathProject, std::string pathTarget, std::string dataFilename); // input file with only mass data for each particle
    
    AerosolPhase(int npTot);
    AerosolPhase(int npTot, std::vector<double> massVector);
    AerosolPhase(int npTot, std::vector<double> massVector, std::vector<double> cVector);
    
    AerosolPhase(Grid* gridPtr, int npTot, int bin, double initNvT); // Initial population in one single bin (randomly distributed within this bin).
    
    int getNpTot();
    
    // access GasParticles
    GasParticle* getGasPartPtr(int i);
    
    std::vector<GasParticle*> getRankedPartPtr();
    
    
    double getCi(int i);
    void setCi(int i, double c);
    double getInitNvT();
    
    double getMass(int i);
    void setMass(int i, double mass);
    void addMass(int i, double deltaMass);
    
    //int countNpl(double mInf, double mSup);
    // not used anymore. count done directly in Psd beacause mAvgVar is now calculated at the same time
    
protected:
    std::vector<GasParticle> m_partVector;
    double m_initNvT;
    
};

#endif /* AerosolPhase_hpp */
