//
//  AgglomerationTensor.cpp
//  npsSolver
//
//  Created by Alexandre Bouaniche on 25/11/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#include "AgglomerationTensor.hpp"

using namespace std;

AgglomerationTensor::AgglomerationTensor(Psd* psdPtr) : m_psdPtr(psdPtr)
{
    setgCoagTensor();
    setavgMassiTensor();
    setavgMassjTensor();
}

double AgglomerationTensor::gCoagValue(int i, int j, int k) // i + j -> k with i <= j and j <= k + 1 ? But with j = k+1 gCoagValue = 0 so we could do only for i <= j <= k
{
    double gCoagVal(0.0);
    
    double mInfi = m_psdPtr->getmInf(i);
    double mSupi = m_psdPtr->getmSup(i);
    double mInfj = m_psdPtr->getmInf(j);
    double mSupj = m_psdPtr->getmSup(j);
    double mSupk = m_psdPtr->getmSup(k);
    double mInfk = m_psdPtr->getmInf(k);
    int nBins = m_psdPtr->getSize();
    int binMax = nBins -1;
    double mMax = m_psdPtr->getmSup(binMax);
    
    double mCenter = ( (mSupi+mSupj) - (mInfi + mInfj) ) /2.0 + (mInfi + mInfj);
    if(mCenter > mMax)
    {
        gCoagVal = 0.0;
    }
    else
    {
        int l = findBin(mCenter);
        
        if(k<(l-1) || k>(l+1))
        {
            gCoagVal = 0.0;
        }
        else if(k<0 || k>binMax)
        {
            gCoagVal = 0.0;
        }
        else if(k==(l-1))
        {
            // check on second integral of the numerator equivalent to mInfi + mInfj > mSupk
            if((mSupk-mInfi)<=mInfj)
            {
                gCoagVal = 0.0;
            }
            else
            {
                double min = mSupi;
                if( (mSupk-mInfj)<mSupi )
                {
                    min = mSupk - mInfj;
                }
                
                double numerator = (min - mInfi) * (mSupk - mInfj) - 0.5 * ( pow(min, 2.0) - pow(mInfi, 2.0) );
                double denominator = (mSupj - mInfj) * (mSupi - mInfi);
                
                gCoagVal = numerator / denominator;
            }
            
            
        }
        else if(k==l)
        {
            // numerator = min(num1; num2; num3; num4)
            double num1 = (mSupi - mInfi) * (mSupj - mInfj);
            double num2 = (mSupi - mInfi) * (mSupj - mInfk) + 0.5 * ( pow(mSupi, 2.0) - pow(mInfi, 2.0)  );
            double num3 = (mSupi - mInfi) * (mSupk - mInfj) - 0.5 * ( pow(mSupi, 2.0) - pow(mInfi, 2.0) );
            double num4 = (mSupi - mInfi) * (mSupk - mInfk);
            
            double numerator = num1;
            if(num2<numerator)
            {
                numerator = num2;
            }
            
            if(num3<numerator)
            {
                numerator = num3;
            }
            if(num4<numerator)
            {
                numerator = num4;
            }
            
            double denominator = (mSupj - mInfj) * (mSupi - mInfi);
            gCoagVal = numerator / denominator;
            
        }
        else if(k==(l+1))
        {
            // check integrals from bornInf to bornSup with bornInf < bornSup
            
            if(mSupj <= (mInfk - mSupi))  // second integral of the numerator. equivalent to mSupj + mSupi < mInfk
            {
                gCoagVal = 0.0;
            }
            else
            {
                //double max = mInfk - mSupi;  erreur D'aubagnac
                double max = mInfk - mSupj;
                if(mInfi>max)
                {
                    max = mInfi;
                }
                
                double numerator = (mSupi - max) * (mSupj - mInfk) + 0.5 * ( pow(mSupi, 2.0) - pow(max, 2.0)  );
                double denominator = (mSupj - mInfj) * (mSupi - mInfi);
                gCoagVal = numerator / denominator;
            }
            
        }
        else
        {
            cout << "ERROR gCoagValue" << endl;
        }
    }
    
    return gCoagVal;
}

double AgglomerationTensor::miCoagValue(int i, int j, int k)
{
    double massiCoagValue(0.0);
    
    double mInfi = m_psdPtr->getmInf(i);
    double mSupi = m_psdPtr->getmSup(i);
    double mInfj = m_psdPtr->getmInf(j);
    double mSupj = m_psdPtr->getmSup(j);
    double mSupk = m_psdPtr->getmSup(k);
    double mInfk = m_psdPtr->getmInf(k);
    
    int nBins = m_psdPtr->getSize();
    int binMax = nBins -1;
    double mMax = m_psdPtr->getmSup(binMax);
    
    double mCenter = ( (mSupi+mSupj) - (mInfi + mInfj) ) /2.0 + (mInfi + mInfj);
    
    if(mCenter > mMax)
    {
        massiCoagValue = 0.0;
    }
    else
    {
        
        int l = findBin(mCenter);
        
        if(k<(l-1) || k>(l+1))
        {
            massiCoagValue = 0.0;
        }
        else if(k<0 || k>binMax)
        {
            massiCoagValue = 0.0;
        }
        else if(k==(l-1))
        {
            // check on second integral of the numerator equivalent to mInfi + mInfj > mSupk
            if((mSupk-mInfi)<=mInfj)
            {
                massiCoagValue = 0.0;
            }
            else
            {
                double min = mSupk - mInfj;
                if(mSupi < min)
                {
                    min = mSupi;
                }
                
                double numerator = 0.5 * (mSupk - mInfj) * ( pow(min, 2.0) - pow(mInfi, 2.0)) - 1.0/3.0 * (pow(min, 3.0) - pow(mInfi, 3.0));
                
                //double denominator = (mSupj - mInfj) * (mSupi - mInfi);
                double denominator = numeratorgCoagLminusOne(i, j, k);  // new denominator
                
                massiCoagValue = numerator/denominator;
            }
            
            
        }
        else if(k==l)
        {
            double num1 = (mSupj - mInfj) * 0.5 * ( pow(mSupi, 2.0) - pow(mInfi, 2.0) );
            double num2 = (mSupj - mInfk) * 0.5 * (pow(mSupi, 2.0) - pow(mInfi, 2.0) ) + 1.0/3.0 * (pow(mSupi, 3.0) - pow(mInfi, 3.0) );
            double num3 = (mSupk - mInfj) * 0.5 * (pow(mSupi, 2.0) - pow(mInfi, 2.0) ) - 1.0/3.0 * (pow(mSupi, 3.0)- pow(mInfi, 3.0) );
            double num4 = (mSupk - mInfk) * 0.5 * (pow(mSupi, 2.0) - pow(mInfi, 2.0) );
            
            double numerator = num1;
            if(num2<numerator)
            {
                numerator = num2;
            }
            
            if(num3<numerator)
            {
                numerator = num3;
            }
            if(num4<numerator)
            {
                numerator = num4;
            }
            
            //double denominator = (mSupj - mInfj) * (mSupi - mInfi);
            double denominator = numeratorgCoagL(i, j, k);
            
            massiCoagValue = numerator / denominator;
        }
        else if(k==(l+1))
        {
            if(mSupj <= (mInfk - mSupi))  // second integral of the numerator. equivalent to mSupj + mSupi < mInfk
            {
                massiCoagValue = 0.0;
            }
            else
            {
                //double max = mInfk - mSupi; erreur D'aubagnac
                double max = mInfk - mSupj;
                if(mInfi > max)
                {
                    max = mInfi;
                }
                
                double numerator = 0.5 * (mSupj - mInfk) * ( pow(mSupi, 2.0) - pow(max, 2.0) ) + 1.0/3.0 * (pow(mSupi, 3.0) - pow(max, 3.0) );
                
                //double denominator = (mSupj - mInfj) * (mSupi - mInfi);
                double denominator = numeratorgCoagLplusOne(i, j, k);
                
                massiCoagValue = numerator/denominator;
            }
            
        }
        
    }
    
    
    return massiCoagValue;
}



double AgglomerationTensor::mjCoagValue(int i, int j, int k)
{
    double massjCoagValue(0.0);
    
    double mInfi = m_psdPtr->getmInf(i);
    double mSupi = m_psdPtr->getmSup(i);
    double mInfj = m_psdPtr->getmInf(j);
    double mSupj = m_psdPtr->getmSup(j);
    double mSupk = m_psdPtr->getmSup(k);
    double mInfk = m_psdPtr->getmInf(k);
    
    int nBins = m_psdPtr->getSize();
    int binMax = nBins -1;
    double mMax = m_psdPtr->getmSup(binMax);
    
    double mCenter = ( (mSupi+mSupj) - (mInfi + mInfj) ) /2.0 + (mInfi + mInfj);
    
    if(mCenter > mMax)
    {
        massjCoagValue = 0.0;
    }
    else
    {
        int l = findBin(mCenter);
        
        if(k<(l-1) || k>(l+1))
        {
            massjCoagValue = 0.0;
        }
        else if(k<0 || k>binMax)
        {
            massjCoagValue = 0.0;
        }
        else if(k == (l-1))
        {
            // check on second integral of the numerator equivalent to mInfi + mInfj > mSupk
            if((mSupk-mInfi)<=mInfj)
            {
                massjCoagValue = 0.0;
            }
            else
            {
                double min = mSupk - mInfj;
                if(mSupi < min)
                {
                    min = mSupi;
                }
                
                double numerator = 0.5 * ( pow(mSupk, 2.0) - pow(mInfj, 2.0) ) * (min - mInfi) + 1.0/6.0 * (pow(min, 3.0) - pow(mInfi, 3.0) ) - 0.5 * mSupk * ( pow(min, 2.0) - pow(mInfi, 2.0) );
                //double denominator = (mSupj - mInfj) * (mSupi - mInfi);
                double denominator = numeratorgCoagLminusOne(i, j, k);
                
                massjCoagValue = numerator / denominator;
            }
            
        }
        else if(k == l)
        {
            double num1 = 0.5 * (pow(mSupj, 2.0) - pow(mInfj, 2.0) ) * (mSupi - mInfi);
            
            double num2 = 0.5 * (pow(mSupj, 2.0) - pow(mInfk, 2.0) ) * (mSupi - mInfi) - 1.0/6.0 * (pow(mSupi, 3.0) - pow(mInfi, 3.0) ) + 0.5 * mInfk * (pow(mSupi, 2.0) - pow(mInfi, 2.0) );
            
            double num3 = 0.5 * (pow(mSupk, 2.0) - pow(mInfk, 2.0) ) * (mSupi - mInfi) -0.5 * (mSupk - mInfk) * (pow(mSupi, 2.0) - pow(mInfi, 2.0) );
            
            double num4 = 0.5 * (pow(mSupk, 2.0) - pow(mInfj, 2.0)) * (mSupi - mInfi) + 1.0/6.0 * (pow(mSupi, 3.0) - pow(mInfi, 3.0)) - 0.5 * mSupk * (pow(mSupi, 2.0) - pow(mInfi, 2.0));
            
            double numerator = num1;
            
            if(num2<numerator)
            {
                numerator = num2;
            }
            
            if(num3<numerator)
            {
                numerator = num3;
            }
            if(num4<numerator)
            {
                numerator = num4;
            }
            
            //double denominator = (mSupj - mInfj) * (mSupi - mInfi);
            double denominator = numeratorgCoagL(i, j, k);
            
            massjCoagValue = numerator / denominator;
        }
        else if(k == (l+1))
        {
            if(mSupj <= (mInfk - mSupi))  // second integral of the numerator. equivalent to mSupj + mSupi < mInfk
            {
                massjCoagValue = 0.0;
            }
            else
            {
                //double max = mInfk - mSupi; ancien, erreur D'Aubagnac
                double max = mInfk - mSupj;
                if(mInfi > max)
                {
                    max = mInfi;
                }
                
                double numerator = 0.5 * (pow(mSupj, 2.0) - pow(mInfk, 2.0)) * (mSupi - max) - 1.0/6.0 * (pow(mSupi, 3.0) - pow(max, 3.0)) + 0.5 * mInfk * (pow(mSupi, 2.0) - pow(max, 2.0));
                
                //double denominator = (mSupj - mInfj) * (mSupi - mInfi);
                double denominator = numeratorgCoagLplusOne(i, j, k);
                
                massjCoagValue = numerator / denominator;
            }
        }
        
    }
    
    return massjCoagValue;
}



int AgglomerationTensor::findBin(double m)
{
    int i(0);
    int bin(0);
    int nBins = m_psdPtr->getSize();
    int iMax = nBins-1;
    double mSupMax = m_psdPtr->getmSup(iMax);
    if(m<=mSupMax)
    {
        double mSup = m_psdPtr->getmSup(i);
        while(m>mSup)
        {
            i++;
            mSup = m_psdPtr->getmSup(i);
        }
        bin = i;
    }
    else
    {
        bin = 0;
        cout << "ERROR findBin, mass looked for > mMax" << endl;
    }
    return bin;
}

void AgglomerationTensor::setgCoagTensor()
{
    int i(0);
    int j(0);
    int k(0);
    int nBins = m_psdPtr->getSize();
    
    vector<vector<vector<double> > > tensor; // vector(i) containing the vectori
    
    for(i=0; i<nBins; i++)
    {
        vector<vector<double> > vectori; // i fixed. vector(j) containing the vectorij.
        
        for(j=0; j<nBins; j++)
        {
            vector<double> vectorij;  // i and j fixed, vector(k)
            
            for(k=0; k<nBins; k++)
            {
                double value(0.0);
                if(i<=j)
                {
                    value = gCoagValue(i, j, k);  // because gCoagValue calculated for i <=j
                }
                else
                {
                    value = gCoagValue(j, i, k); // because gCoagValue calculated for i <=j
                }
                
                vectorij.push_back(value);
            }
            
            vectori.push_back(vectorij);
            
        }
        tensor.push_back(vectori);
    }
    
    m_gCoagTensor = tensor;
}


void AgglomerationTensor::setavgMassiTensor()
{
    int i(0);
    int j(0);
    int k(0);
    int nBins = m_psdPtr->getSize();
    
    vector<vector<vector<double> > > tensor; // vector(i) containing the vectori
    
    for(i=0; i<nBins; i++)
    {
        vector<vector<double> > vectori; // i fixed. vector(j) containing the vectorij.
        
        for(j=0; j<nBins; j++)
        {
            vector<double> vectorij;  // i and j fixed, vector(k)
            
            for(k=0; k<nBins; k++)
            {
                double value(0.0);
                
                if((i<=j) && (j<=k) )
                {
                    value = miCoagValue(i, j, k);  // only for i <= j
                }
                else
                {
                    value = 0.0;
                }
                vectorij.push_back(value);
            }
            
            vectori.push_back(vectorij);
            
        }
        tensor.push_back(vectori);
    }
    
    m_avgMassiTensor = tensor;
}


void AgglomerationTensor::setavgMassjTensor()
{
    int i(0);
    int j(0);
    int k(0);
    int nBins = m_psdPtr->getSize();
    
    vector<vector<vector<double> > > tensor; // vector(i) containing the vectori
    
    for(i=0; i<nBins; i++)
    {
        vector<vector<double> > vectori; // i fixed. vector(j) containing the vectorij.
        
        for(j=0; j<nBins; j++)
        {
            vector<double> vectorij;  // i and j fixed, vector(k)
            
            for(k=0; k<nBins; k++)
            {
                double value(0.0);
                if((i<=j) && (j<=k) )
                {
                    value = mjCoagValue(i, j, k);
                }
                else
                {
                    value = 0.0;
                }
                
                vectorij.push_back(value);
            }
            
            vectori.push_back(vectorij);
            
        }
        tensor.push_back(vectori);
    }
    
    m_avgMassjTensor = tensor;
}

double AgglomerationTensor::getgCoagValue(int i, int j, int k)
{
    return m_gCoagTensor[i][j][k];
}

double AgglomerationTensor::getmAvgFromiOrigin(int i, int j, int k)
{
    return m_avgMassiTensor[i][j][k];
}

double AgglomerationTensor::getmAvgFromjOrigin(int i, int j, int k)
{
    return m_avgMassjTensor[i][j][k];
}

double AgglomerationTensor::numeratorgCoagLminusOne(int i, int j, int k)
{
    double gCoagVal(0.0);
    
    double mInfi = m_psdPtr->getmInf(i);
    double mSupi = m_psdPtr->getmSup(i);
    double mInfj = m_psdPtr->getmInf(j);
    double mSupj = m_psdPtr->getmSup(j);
    double mSupk = m_psdPtr->getmSup(k);
    //double mInfk = m_psdPtr->getmInf(k);
    int nBins = m_psdPtr->getSize();
    int binMax = nBins -1;
    double mMax = m_psdPtr->getmSup(binMax);
    
    double mCenter = ( (mSupi+mSupj) - (mInfi + mInfj) ) /2.0 + (mInfi + mInfj);
    if(mCenter > mMax)
    {
        gCoagVal = 0.0;
    }
    else
    {
        int l = findBin(mCenter);
        
        if(k<(l-1) || k>(l+1))
        {
            gCoagVal = 0.0;
        }
        else if(k<0 || k>binMax)
        {
            gCoagVal = 0.0;
        }
        else if(k==(l-1))
        {
            // check on second integral of the numerator equivalent to mInfi + mInfj > mSupk
            if((mSupk-mInfi)<mInfj)
            {
                gCoagVal = 0.0;
            }
            else
            {
                double min = mSupi;
                if( (mSupk-mInfj)<mSupi )
                {
                    min = mSupk - mInfj;
                }
                
                double numerator = (min - mInfi) * (mSupk - mInfj) - 0.5 * ( pow(min, 2.0) - pow(mInfi, 2.0) );
                double denominator = (mSupj - mInfj) * (mSupi - mInfi);
                
                gCoagVal = numerator / denominator;
            }
            
            
        }
    }
    double denominator = (mSupj - mInfj) * (mSupi - mInfi);
    double numeratorFinal = gCoagVal * denominator;
    return numeratorFinal;
}

double AgglomerationTensor::numeratorgCoagL(int i, int j, int k)
{
    double gCoagVal(0.0);
    
    double mInfi = m_psdPtr->getmInf(i);
    double mSupi = m_psdPtr->getmSup(i);
    double mInfj = m_psdPtr->getmInf(j);
    double mSupj = m_psdPtr->getmSup(j);
    double mSupk = m_psdPtr->getmSup(k);
    double mInfk = m_psdPtr->getmInf(k);
    int nBins = m_psdPtr->getSize();
    int binMax = nBins -1;
    double mMax = m_psdPtr->getmSup(binMax);
    
    double mCenter = ( (mSupi+mSupj) - (mInfi + mInfj) ) /2.0 + (mInfi + mInfj);
    if(mCenter > mMax)
    {
        gCoagVal = 0.0;
    }
    else
    {
        int l = findBin(mCenter);
        
        if(k<(l-1) || k>(l+1))
        {
            gCoagVal = 0.0;
        }
        else if(k<0 || k>binMax)
        {
            gCoagVal = 0.0;
        }
        else if(k==l)
        {
            // numerator = min(num1; num2; num3; num4)
            double num1 = (mSupi - mInfi) * (mSupj - mInfj);
            double num2 = (mSupi - mInfi) * (mSupj - mInfk) + 0.5 * ( pow(mSupi, 2.0) - pow(mInfi, 2.0)  );
            double num3 = (mSupi - mInfi) * (mSupk - mInfj) - 0.5 * ( pow(mSupi, 2.0) - pow(mInfi, 2.0) );
            double num4 = (mSupi - mInfi) * (mSupk - mInfk);
            
            double numerator = num1;
            if(num2<numerator)
            {
                numerator = num2;
            }
            
            if(num3<numerator)
            {
                numerator = num3;
            }
            if(num4<numerator)
            {
                numerator = num4;
            }
            
            double denominator = (mSupj - mInfj) * (mSupi - mInfi);
            gCoagVal = numerator / denominator;
            
        }
        
    }
    double denominator = (mSupj - mInfj) * (mSupi - mInfi);
    double numeratorFinal = gCoagVal * denominator;
    return numeratorFinal;
}


double AgglomerationTensor::numeratorgCoagLplusOne(int i, int j, int k)
{
    double gCoagVal(0.0);
    
    double mInfi = m_psdPtr->getmInf(i);
    double mSupi = m_psdPtr->getmSup(i);
    double mInfj = m_psdPtr->getmInf(j);
    double mSupj = m_psdPtr->getmSup(j);
    //double mSupk = m_psdPtr->getmSup(k);
    double mInfk = m_psdPtr->getmInf(k);
    int nBins = m_psdPtr->getSize();
    int binMax = nBins -1;
    double mMax = m_psdPtr->getmSup(binMax);
    
    double mCenter = ( (mSupi+mSupj) - (mInfi + mInfj) ) /2.0 + (mInfi + mInfj);
    if(mCenter > mMax)
    {
        gCoagVal = 0.0;
    }
    else
    {
        int l = findBin(mCenter);
        
        if(k<(l-1) || k>(l+1))
        {
            gCoagVal = 0.0;
        }
        else if(k<0 || k>binMax)
        {
            gCoagVal = 0.0;
        }
        else if(k==(l+1))
        {
            // check integrals from bornInf to bornSup with bornInf < bornSup
            
            if(mSupj < (mInfk - mSupi))  // second integral of the numerator. equivalent to mSupj + mSupi < mInfk
            {
                gCoagVal = 0.0;
            }
            else
            {
                //double max = mInfk - mSupi;  erreur D'aubagnac
                double max = mInfk - mSupj;
                if(mInfi>max)
                {
                    max = mInfi;
                }
                
                double numerator = (mSupi - max) * (mSupj - mInfk) + 0.5 * ( pow(mSupi, 2.0) - pow(max, 2.0)  );
                double denominator = (mSupj - mInfj) * (mSupi - mInfi);
                gCoagVal = numerator / denominator;
            }
            
        }
        
    }
    double denominator = (mSupj - mInfj) * (mSupi - mInfi);
    double numeratorFinal = gCoagVal * denominator;
    return numeratorFinal;
}
