//
//  Bin.cpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 28/08/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#include "Bin.hpp"
#include "GridCell.hpp"

#include <iostream>
#include <string>
#include <vector>
#include <cmath>

using namespace std;

Bin::Bin(double mConst) : GridCell(mConst), m_np(0), m_nv(0.0), m_nvResidual(0.0), m_nvStochastic(0.0), m_mAvgVar(mConst), m_mAvgVarStoch(mConst), m_mAvgVarR(mConst)
{
    
}

Bin::Bin(double mConst, double mAvg, double mInf, double mSup) : GridCell(mConst, mAvg, mInf, mSup) , m_np(0), m_nv(0.0), m_nvResidual(0.0), m_nvStochastic(0.0), m_mAvgVar(mAvg), m_mAvgVarStoch(mAvg), m_mAvgVarR(mAvg)
{
    
}


Bin::Bin(GridCell gridCell, int np) : GridCell(gridCell.getmConst(), gridCell.getmAvg(), gridCell.getmInf(), gridCell.getmSup()), m_np(np), m_nv(0.0), m_nvResidual(0.0), m_nvStochastic(0.0), m_mAvgVar(gridCell.getmAvg()), m_mAvgVarStoch(gridCell.getmAvg()), m_mAvgVarR(gridCell.getmAvg())
{
    
}


Bin::Bin(GridCell gridCell, int np, double nv) : GridCell(gridCell.getmConst(), gridCell.getmAvg(), gridCell.getmInf(), gridCell.getmSup()), m_np(np), m_nv(nv), m_nvResidual(0.0), m_nvStochastic(0.0), m_mAvgVar(gridCell.getmAvg()), m_mAvgVarStoch(gridCell.getmAvg()), m_mAvgVarR(gridCell.getmAvg())
{
    
}


Bin::Bin(GridCell gridCell, double nvResidual) : GridCell(gridCell.getmConst(), gridCell.getmAvg(), gridCell.getmInf(), gridCell.getmSup()), m_np(0), m_nv(0.0), m_nvResidual(nvResidual), m_nvStochastic(0.0), m_mAvgVar(gridCell.getmAvg()), m_mAvgVarStoch(gridCell.getmAvg()), m_mAvgVarR(gridCell.getmAvg())
{
    m_nv = m_nvStochastic + m_nvResidual;
}


int Bin::getNp()
{
    return m_np;
}

double Bin::getNv()
{
    m_nv = m_nvStochastic + m_nvResidual;
    return m_nv;
}

double Bin::getNvResidual()
{
    return m_nvResidual;
}


void Bin::setNp(int np)
{
    m_np = np;
}


void Bin::setNvResidual(double nvResidual)
{
    m_nvResidual = nvResidual;
}

void Bin::setNvStochastic(double nvStochastic)
{
    m_nvStochastic = nvStochastic;
}

void Bin::addNvResidual(double deltaNvResidual)
{
    m_nvResidual += deltaNvResidual;
}

void Bin::showBin()
{
    cout << endl;
    cout << "mAvg   np   nv   nvResidual" << endl;
    cout << m_mAvg << "   " << m_np << "   " << m_nv << "   " << m_nvResidual<< endl;
}

void Bin::setmAvgVar(double mAvgVar)
{
    m_mAvgVar = mAvgVar;
}

double Bin::getmAvgVar()
{
    return m_mAvgVar;
}

void Bin::setmAvgVarStoch(double mAvgVar)
{
    m_mAvgVarStoch = mAvgVar;
}

double Bin::getmAvgVarStoch()
{
    return m_mAvgVarStoch;
}

void Bin::setmAvgVarR(double mAvgVar)
{
    m_mAvgVarR = mAvgVar;
}

double Bin::getmAvgVarR()
{
    return m_mAvgVarR;
}

