//
//  Bin.hpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 28/08/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#ifndef Bin_hpp
#define Bin_hpp

#include <stdio.h>
#include <vector>

#include "GridCell.hpp"

class Bin : public GridCell
{
public:
    Bin(double mConst);
    Bin(double mConst, double mAvg, double mInf, double mSup);
    
    Bin(GridCell gridCell, int np);
    Bin(GridCell gridCell, int np, double nv);
    
    // case sectional
    Bin(GridCell gridCell, double nvResidual);
    
    
    int getNp();
    double getNv();
    double getNvResidual();
    
    void setNp(int np);
    void setNvResidual(double nvResidual);
    void setNvStochastic(double nvStochastic);
    
    void addNvResidual(double deltaNvResidual);
    
    void showBin();
    
    void setmAvgVar(double mAvgVar);
    double getmAvgVar();
    void setmAvgVarStoch(double mAvgVar);
    double getmAvgVarStoch();
    void setmAvgVarR(double mAvgVar);
    double getmAvgVarR();
    
    
protected:
    
    int m_np;  // [stochastic particles]
    double m_nv;  // [real particles / gas volume]
    double m_nvResidual; // [part/ gas volume]. correction with pbeMethod == "npResidual"
    double m_nvStochastic;  // [part/ gas volume]. from counting np and from nvT.
    double m_mAvgVar; // [g]. Average mass per particle of the stochastic particles (AND NvR !! ) inside the bin. Made to be used for kumar agglo and for hybrid sectional growth.
    double m_mAvgVarStoch; // [g]. Average mass per particle of the stochastic particles (only) inside the bin. Made to be used for kumar agglo and for hybrid sectional growth.
    double m_mAvgVarR; // [g]. Average mass per particle of residual term inside a section.
    
};



#endif /* Bin_hpp */
