//
//  GridCell.cpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 28/08/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#include "GridCell.hpp"

#include <iostream>
#include <string>
#include <vector>
#include <cmath>

using namespace std;

const double GridCell::rhoSoot = 1.8; // [g/cm^3]
const double GridCell::pi = 3.14159265;

GridCell::GridCell(double mConst) : m_mConst(mConst)
{
    
}

GridCell::GridCell(double mConst, double mAvg, double mInf, double mSup) : m_mConst(mConst), m_mAvg(mAvg), m_mInf(mInf), m_mSup(mSup)
{
    
}


double GridCell::getmConst()
{
    return m_mConst;
}

double GridCell::getmAvg()
{
    return m_mAvg;
}

double GridCell::getmInf()
{
    return m_mInf;
}

double GridCell::getmSup()
{
    return m_mSup;
}

void GridCell::setmConst(double mConst)
{
    m_mConst = mConst;
}

void GridCell::setmAvg(double mAvg)
{
    m_mAvg = mAvg;
}

void GridCell::setmInf(double mInf)
{
    m_mInf = mInf;
}

void GridCell::setmSup(double mSup)
{
    m_mSup = mSup;
}

double GridCell::getDfromMassNano(double m)  // for sphere
{
    double v = m/rhoSoot;   // [cm3]  m_mAvg in [g]
    double oneThird = 1.0/3.0;
    double d3 = v*6.0/pi;
    double dcm = pow(d3, oneThird);  // [in cm]
    double dnm = dcm * 1.0e7;   // [in nm]
    return dnm;
}

double GridCell::getSfromMass(double m)  // for sphere
{
    double v = m/rhoSoot;  // in [cm3]    m_mAvg in [g]
    double oneThird = 1.0/3.0;
    double d3 = v*6.0/pi;
    double dcm = pow(d3, oneThird);  // [in cm]
    double surf = pi * pow(dcm, 2.0); // [in cm2]
    return surf;
    
}

double GridCell::getdAvgNano()
{
    double d(0.0);
    
    d = getDfromMassNano(m_mAvg);
    return d;
}

double GridCell::getdInfNano()
{
    
    double d(0.0);
    d = getDfromMassNano(m_mInf);
    return d;
}

double GridCell::getdSupNano()
{
    double d(0.0);
    d = getDfromMassNano(m_mSup);
    return d;
}

double GridCell::getsAvg()
{
    double s(0.0);
    s = getSfromMass(m_mAvg);
    return s;
}

double GridCell::getsInf()
{
    double s(0.0);
    s = getSfromMass(m_mInf);
    return s;
}

double GridCell::getsSup()
{
    double s(0.0);
    s = getSfromMass(m_mSup);
    return s;
}



