//
//  GridCell.hpp
//  soot_stochastic
//
//  Created by Alexandre Bouaniche on 28/08/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#ifndef GridCell_hpp
#define GridCell_hpp

#include <stdio.h>
#include <vector>
#include <string>


// Object that represents one "empty" cell of the grid in the space of sizes (mass).
// different attributes representing the mass of the cell "by construction" the corresponding
// average mass of the cell and the inf and sup born masses

class GridCell
{
public:
    
    //Creator
    GridCell(double mConst);
    GridCell(double mConst, double mAvg, double mInf, double mSup);
    
    // mass in g
    double getmConst();
    double getmAvg();
    double getmInf();
    double getmSup();
    
    void setmConst(double mConst);
    void setmAvg(double mAvg);
    void setmInf(double mInf);
    void setmSup(double mSup);
    
    
    // spheres
    
    // diameters: Here from sphere. Aggregate shape considered in SootSourceTerm
    // diameter in nanoMeters here because used for Nd. For SootSourceTerms, diameters in cm
    double getDfromMassNano(double m); // in nanometer
    double getdAvgNano();
    double getdInfNano();
    double getdSupNano();
    
    // Surface: Here from sphere. Aggregate shape considered in SootSourceTerm
    // Surface in in cm2
    double getSfromMass(double m);  // in cm2
    double getsAvg();
    double getsInf();
    double getsSup();
    
    
protected:
    double m_mConst;  // mass "by construction" of the grid. For example corresponds to a geometric grid series pattern
    double m_mAvg;
    double m_mInf;
    double m_mSup;
    
    static const double rhoSoot;
    static const double pi;
    
};

#endif /* GridCell_hpp */
