//
//  GrowthPark.cpp
//  npsSolver
//
//  Created by Alexandre Bouaniche on 29/03/2018.
//  Copyright © 2018 Alexandre Bouaniche. All rights reserved.
//

#include "GrowthPark.hpp"

using namespace std;

GrowthPark::GrowthPark(Psd* psdPtr, SootSourceTerm* sourcePtr, string solverType, double fs) : m_psdPtr(psdPtr), m_fs(fs), m_solverType(solverType), m_sourcePtr(sourcePtr)
{
    
}


double GrowthPark::bi(int bini)
{
    double fs = m_fs;
    
    double dlnN = dln(bini);
    double bi(0.0);
    
    if(dlnN > 10.0)  // asymptotic values for dlnN > 10 or dlnN < -10
    {
        //bi = -1.0;  // -1 if fs = 2. In general (see below):
        bi = -1.0 / (fs-1.0);
    }
    else if(dlnN < -10.0)
    {
        //bi = 2.0; // 2.0 if fs = 2. In general (see below):
        bi = fs / (fs - 1.0);
    }
    else
    {
        
        if(dlnN <= 0.0)
        {
            bi = - fs / (fs - 1.0) * erf(1.0/4.0 * dlnN);
        }
        else
        {
            bi = - 1.0 / (fs - 1.0) * erf(1.0/4.0 * dlnN);
        }
    }
    
    // test force upwind growth on bin0  !!!
    
    if(bini == 0)
    {
        //bi = -1.0; // -1 if fs = 2
        bi = -1.0 / (fs-1.0);
    }
    
    
    return bi;
}

double GrowthPark::ai(int bini)
{
    double ai(0.0);
    double fs = m_fs;
    ai = ( fs - bi(bini)* (fs - 1.0) ) / (pow(fs, 2.0) - 1.0);
    return ai;
}

double GrowthPark::ci(int bini)
{
    double ci(0.0);
    
    ci = -(ai(bini) + bi(bini));
    return ci;
}

double GrowthPark::dln(int bini)  // test: calculate dln first order upwind instead of second-order central scheme
{
    double dlnN = 0.0;
    int binMax = m_psdPtr->getSize() - 1;
    double limitNullPos = 1.0e-20;
    
    //Bin* binPtr = m_psdPtr->getBinPtr(bini);
    
    
    if(bini == 0)
    {
        //double mAvgi = m_psdPtr->getmAvg(bini);
        //double mAvgiPlusOne = m_psdPtr->getmAvg(bini+1);
        double mAvgVari = m_psdPtr->getmAvgVar(bini);
        double mAvgVariPlusOne = m_psdPtr->getmAvgVar(bini+1);
        
        double Nvi = m_psdPtr->getNv(bini);
        double NviPlusOne = m_psdPtr->getNv(bini + 1);
        
        
        // asymptotic limits for dlnN to be bounded
        // put a condition so that if Nv << 1 like 1.0e-50 for example then do not calculate log(Nvi). return value 10 or -10 for dlnN (extreme values giving asymptotic values for bi)
        
        if(Nvi < limitNullPos && NviPlusOne > limitNullPos)
        {
            dlnN = 10.0;
        }
        else if(Nvi > limitNullPos && NviPlusOne < limitNullPos)
        {
            dlnN = -10.0;
        }
        else if(Nvi < limitNullPos && NviPlusOne < limitNullPos)
        {
            dlnN = 0.0;
        }
        else
        {
            //dlnN = ( log(NviPlusOne) - log(Nvi) ) / (log(mAvgiPlusOne) - log(mAvgi));
            dlnN = ( log(NviPlusOne) - log(Nvi) ) / (log(mAvgVariPlusOne) - log(mAvgVari));
        }
        
        //dlnN = 10.0; // for first and last bins, asymptotic limits to upwind
        
    }
    
    else if(bini == binMax)
    {
        
        //double mAvgi = m_psdPtr->getmAvg(bini);
        //double mAvgiMinusOne = m_psdPtr->getmAvg(bini-1);
        double mAvgVari = m_psdPtr->getmAvgVar(bini);
        double mAvgVariMinusOne = m_psdPtr->getmAvgVar(bini-1);
        
        double Nvi = m_psdPtr->getNv(bini);
        double NviMinusOne = m_psdPtr->getNv(bini -1);
        
        // asymptotic limits for dlnN to be bounded
        //if(NvRi < limitNullPos && NvRi > limitNullNeg)
        
        
        if(Nvi < limitNullPos && Nvi < limitNullPos)
        {
            dlnN = 0.0;
        }
        else if(Nvi > limitNullPos && NviMinusOne < limitNullPos)
        {
            dlnN = 10.0;
        }
        else if(Nvi < limitNullPos && NviMinusOne > limitNullPos)
        {
            dlnN = -10.0;
        }
        else
        {
            //dlnN = ( log(Nvi) - log(NviMinusOne) ) / (log(mAvgi) - log(mAvgiMinusOne) );
            dlnN = ( log(Nvi) - log(NviMinusOne) ) / (log(mAvgVari) - log(mAvgVariMinusOne) );
            
        }
        
         
        //dlnN = 10.0; // for first and last bins, asymptotic limits to upwind
        
    }
    
    else if(bini > 0 && bini < binMax)
    {
        
        //double mAvgiPlusOne = m_psdPtr->getmAvg(bini+1);
        //double mAvgiMinusOne = m_psdPtr->getmAvg(bini-1);
        //double mAvgi = m_psdPtr->getmAvg(bini);
        
        double mAvgVariPlusOne = m_psdPtr->getmAvgVar(bini+1);
        double mAvgVariMinusOne = m_psdPtr->getmAvgVar(bini-1);
        double mAvgVari = m_psdPtr->getmAvgVar(bini);
        
        double Nvi = m_psdPtr->getNv(bini);
        double NviMinusOne = m_psdPtr->getNv(bini -1);
        double NviPlusOne = m_psdPtr->getNv(bini + 1);
        
        // asymptotic limits for dlnN to be bounded
        if(NviMinusOne < limitNullPos && NviPlusOne > limitNullPos )  // asymptotic dlnN / dlnv ~ + inf
        {
            dlnN = 10.0;
        }
        else if(NviMinusOne > limitNullPos && NviPlusOne < limitNullPos )  // asymptotic dlnN / dlnv ~ - inf
        {
            dlnN = -10.0;
        }
        else if(NviMinusOne < limitNullPos && NviPlusOne < limitNullPos )
        {
            dlnN = 0.0;
        }
        else
        {
            double dlnN2 = (log(NviPlusOne) - log(NviMinusOne)) / ( log(mAvgVariPlusOne) - log(mAvgVariMinusOne) ); //second order centered
            double dlnN1 = (log(Nvi) - log(NviMinusOne)) / ( log(mAvgVari) - log(mAvgVariMinusOne) );  // first order upwind
            
            
            //cout << "GrowthPark::dln dlnN1 = " << dlnN1 << "   dlnN2 = " << dlnN2 << "   log(mAvgVariPlusOne) = " << log(mAvgVariPlusOne) << "   log(mAvgVariMinusOne) = " << log(mAvgVariMinusOne) << endl << endl;
            //cout << "NviPlusOne = " << NviPlusOne << "   NviMinusOne = " << NviMinusOne << "   mAvgVariPlusOne = " << mAvgVariPlusOne << "   mAvgVariMinusOne = " << mAvgVariMinusOne << endl << endl;
            
            
            if(dlnN1 * dlnN2 < 0.0)  // if opposed signs, first order is taken
            {
                dlnN = dlnN1;  // good for case Park3 in order to avoid instabilities but bad for case Park2 because it doesn't allow to recover the right peak
                //dlnN = dlnN2;
            }
            else    // if same sign, second order is taken
            {
                dlnN = dlnN2;
            }
        }
        
        
    }
    else
    {
        cout << "ERROR GrowthPark::dlnN, argument i" << endl;
    }
    
    
    return dlnN;
}



double GrowthPark::g(int bini, double t)
{
    double g = m_sourcePtr->deltamTotGrowthVector(t)[bini];
    
    return g;
}

double GrowthPark::dotNvi(int i, double t)  // [part.cm-3.s-1]
{
    double dotNv(0.0);
    double gi = g(i, t);
    
    int nBins = m_psdPtr->getSize();
    int binMax = nBins - 1;
    
    Bin* binPtr = m_psdPtr->getBinPtr(i);
    
    
    double NvRi = binPtr->getNvResidual();
    
    //cout << "NvRi = " << NvRi << endl;
    
    if(i == 0)   // Before but not anymore: eq (6) and (7) of Park, Rogak, 2004 corrected using mSup and mInf instead of mAvg as described in Ramkrishna and Hounslow. Actually, as eq (12) of Kumar, Ramkrishna 1997 part III.
    {
        Bin* binPlusOnePtr = m_psdPtr->getBinPtr(i+1);
        
        if(gi >= 0.0)  // eq (6)
        {
            // Version full park
            
            double NvRiPlusOne = binPlusOnePtr->getNvResidual();
            
            //double mAvgi = m_psdPtr->getmAvg(i);
            //double mAvgiPlusOne = m_psdPtr->getmAvg(i+1);
            double mAvgVarRi = m_psdPtr->getmAvgVarR(i);
            double mAvgVarRiPlusOne = m_psdPtr->getmAvgVarR(i+1);
            
            //dotNv = bi(i) * g(i) * NvRi / mAvgi + ci(i+1) * g(i+1) * NvRiPlusOne / mAvgiPlusOne;
            dotNv = bi(i) * gi * NvRi / mAvgVarRi + ci(i+1) * g(i+1,t) * NvRiPlusOne / mAvgVarRiPlusOne;
            
            //cout << "check Park: " << "   i = " << i << "   bi = " << bi(i) << "   gi = " << gi << "   NvRi = " << NvRi << "   mAvgVari = " << mAvgVarRi << " ci = " << ci(i+1) << "   gi+1 = " << g(i+1,t) << "   NvRiPlusOne = " << NvRiPlusOne << "   mAvgVariPlusOne = " << mAvgVarRiPlusOne  << endl;
        }
        else
        {
            
            // Version full park
            double NvRiPlusOne = binPlusOnePtr->getNvResidual();
            
            //double mAvgi = m_psdPtr->getmAvg(i);
            //double mAvgiPlusOne = m_psdPtr->getmAvg(i+1);
            double mAvgVarRi = m_psdPtr->getmAvgVarR(i);
            double mAvgVarRiPlusOne = m_psdPtr->getmAvgVarR(i+1);
            
            //dotNv = bi(i) * g(i) * NvRi / mAvgi + ci(i+1) * g(i+1) * NvRiPlusOne / mAvgiPlusOne;
            dotNv = bi(i) * gi * NvRi / mAvgVarRi + ci(i+1) * g(i+1, t) * NvRiPlusOne / mAvgVarRiPlusOne;
        }
        
    }
    else if(i == binMax)
    {
        Bin* binMinusOnePtr = m_psdPtr->getBinPtr(i-1);
        
         // version upwind Hounslow ?
        
        // Version full park
        double NvRiMinusOne = binMinusOnePtr->getNvResidual();
        
        //double mAvgiMinusOne = m_psdPtr->getmAvg(i-1);
        //double mAvgi = m_psdPtr->getmAvg(i);
        double mAvgVarRiMinusOne = m_psdPtr->getmAvgVarR(i-1);
        double mAvgVarRi = m_psdPtr->getmAvgVarR(i);
        
        //dotNv = ai(i-1) * g(i-1) * NvRiMinusOne / mAvgiMinusOne + bi(i) * g(i) * NvRi / mAvgi;
        dotNv = ai(i-1) * g(i-1, t) * NvRiMinusOne / mAvgVarRiMinusOne + bi(i) * gi * NvRi / mAvgVarRi;
        
    }
    else if(i>0 && i<binMax) // eq (9) of Park, Rogak, 2004. Here vi is mAvgi.
    {
        Bin* binMinusOnePtr = m_psdPtr->getBinPtr(i-1);
        Bin* binPlusOnePtr = m_psdPtr->getBinPtr(i+1);
        
        double NvRiMinusOne  = binMinusOnePtr->getNvResidual();
        double NvRiPlusOne = binPlusOnePtr->getNvResidual();
        
        //double mAvgi = m_psdPtr->getmAvg(i);
        //double mAvgiPlusOne = m_psdPtr->getmAvg(i+1);
        //double mAvgiMinusOne = m_psdPtr->getmAvg(i-1);
        
        double mAvgVarRi = m_psdPtr->getmAvgVarR(i);
        double mAvgVarRiPlusOne = m_psdPtr->getmAvgVarR(i+1);
        double mAvgVarRiMinusOne = m_psdPtr->getmAvgVarR(i-1);
        
        //dotNv = ai(i-1) * g(i-1) * NvRiMinusOne / mAvgiMinusOne + bi(i) * g(i) * NvRi / mAvgi + ci(i+1) * g(i+1) * NvRiPlusOne / mAvgiPlusOne;
        dotNv = ai(i-1) * g(i-1, t) * NvRiMinusOne / mAvgVarRiMinusOne + bi(i) * gi * NvRi / mAvgVarRi + ci(i+1) * g(i+1, t) * NvRiPlusOne / mAvgVarRiPlusOne;
        
    }
    else
    {
        cout << "ERROR GrowthPark::dotNvi  bin out of bounds" << endl;
    }

    
    //cout << "GrowthPark::dotNvi dotNv = " << dotNv << "   ai = "
    return dotNv;
}
