//
//  GrowthPark.hpp
//  npsSolver
//
//  Created by Alexandre Bouaniche on 29/03/2018.
//  Copyright © 2018 Alexandre Bouaniche. All rights reserved.
//

#ifndef GrowthPark_hpp
#define GrowthPark_hpp

#include <stdio.h>

#include <vector>
#include <string>
#include <iostream>
#include <cmath>

#include "Psd.hpp"
#include "SootSourceTerm.hpp"

/* class used for fluxes of growth between bins for sectional terms (npResiudal...)
used for G(x) = G0 * x^teta with constant G0 (analytical)
 based on the paper Park, Rogak 2004, Aerosol Science 35
*/

class GrowthPark
{
public:
    GrowthPark(Psd* psdPtr, SootSourceTerm* sourcePtr, std::string solverType, double fs);
    
    double dotNvi(int i, double t);
    double g(int bini, double t); // for case solverType == "analytical" or == "HomogeneousGasPhase"
    
protected:
    Psd* m_psdPtr;
    std::string m_solverType;
    SootSourceTerm* m_sourcePtr;
    
    // if growth term given as G(x) = G0 * x^teta
    /*
    double m_G0;
    double m_teta;
     */
    
    double m_fs; // (grid geometric ratio mAvg(i+1) / mAvg(i)
     
    // protected methods
    // Coefficients for the growth differential equation
    double bi(int bini); // calculated first from psd and fs (Nvi, xi, fs)
    double ai(int bini); // calculated from bi
    double ci(int bini); // calculated from ai and bi
    
    double dln(int bini); // returns d(ln Nvi) / d(ln xi) using the second order central scheme. (as said in Park, Rogak, 2004). ( ln[N(i+1)] - ln[N(i-1)] ) / [ ln(x_{i+1}) - ln(x_{i-1}) ]. For first cell and last cell, respectively only i and i+1 or i and i-1 are used to calculate the derivative
    
    
};
 
#endif /* GrowthPark_hpp */
