//
//  GrowthTensor.hpp
//  npsSolver
//
//  Created by Alexandre Bouaniche on 16/02/2018.
//  Copyright © 2018 Alexandre Bouaniche. All rights reserved.
//

#ifndef GrowthTensor_hpp
#define GrowthTensor_hpp

#include <stdio.h>

#include <vector>
#include <string>
#include <iostream>
#include <cmath>

#include "Psd.hpp"

// class used for fluxes of growth between bins for sectional terms (npResiudal...)
// used for G(x) = G0 * x^teta with constant G0 (analytical)
// for physical problems with soots, dotmGasBin will be calculated directly from omega_r*M_alpha

class GrowthTensor
{
public:
    GrowthTensor(Psd* psdPtr, double G0, double teta, double Malpha, std::string solverType, std::string sectionalMethod);
    
    double getDotNvBin(int i);  // [part.cm-3.s-1]   to add to NvResidual
    
    double getDotmTotBin(int i);  // [g.cm-3.s-1]
    double getDotmOutBin(int i);  // [g.cm-3.s-1]
    double getDotmGasBin(int i);  // [g.cm-3.s-1]
    
    double getDotmOutNegBin(int i);  // [g.cm-3.s-1]
    
    double getG(int i);
    void setGrowthtStep(double growthtStep);
    
protected:
    Psd* m_psdPtr;
    double m_growthtStep;
    
    // if growth term given as G(x) = G0 * x^teta   with Malpha as elementary mass addition
    // for now only flux to bin i+1 (G(x) > 0 and Malpha sufficiently small not to "jump" bins.
    double m_G0;
    double m_teta;
    double m_Malpha;
    std::string m_solverType;
    std::string m_sectionalMethod;
    
    std::vector<double> m_gGasTensor;   // calculated only once at the beginning of the simulation. uses Malpha
    std::vector<double> m_gOutTensor;   // calculated only once at the beginning of the simulation. uses Malpha
    std::vector<double> m_gOutNegTensor;   // calculated only once at the beginning of the simulation. uses Malpha
    
    double gGasCoef(int i);  // calculated at each iteration. (for "2pointGdt" pbeMethod) equivalent of Malpha ~ G * growthtStep
    double gOutCoef(int i);  // calculated at each iteration. (for "2pointGdt" pbeMethod) equivalent of Malpha ~ G * growthtStep
    double gOutNegCoef(int i);  // calculated at each iteration. (for "2pointGdt" pbeMethod) equivalent of Malpha ~ G * growthtStep

    // protected methods
    void setgGasTensor();
    void setgOutTensor();
    void setgOutNegTensor();
    
    double omega_bini(int i); // added for correction with 2pointGdt. Coding directly DLR equation
    double omega_biniPlusOne(int i); // added for correction with 2pointGdt. Coding directly DLR equation
    double mj_jPlusOne(int i);
    
    double omega_r(int i);
    
};

#endif /* GrowthTensor_hpp */
