//
//  NpSolver.hpp
//  npsSolver
//
//  Created by Alexandre Bouaniche on 08/09/2017.
//  Copyright © 2017 Alexandre Bouaniche. All rights reserved.
//

#ifndef NpSolver_hpp
#define NpSolver_hpp

#include <stdio.h>

#include <vector>
#include <iostream>
#include <fstream>
#include <string>
#include <cmath>
#include <sstream>
#include <random>


#include "SootSourceTerm.hpp"
#include "AerosolPhase.hpp"
#include "Psd.hpp"
#include "SootStochasticParticle.hpp"
#include "GasParticle.hpp"
#include "HomogeneousGasPhase.hpp"
#include "TimeStep.hpp"
#include "GrowthPark.hpp"

#include "tools.hpp"

class NpSolver
{
public:
    NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr);
    NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt);

    NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, std::string pathProject, std::string pathTargetPsd, std::string pathTargetGlobal);
    
    NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, std::string pathProject, std::string pathTargetPsd, std::string pathTargetGlobal, std::string solverType);
    
    NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, std::string pathProject, std::string pathTargetPsd, std::string pathTargetGlobal, std::string solverType, TimeStep* timeStepPtr, double tFinal, std::string pbeMethod);
    
    
    NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, std::string pathProject, std::string pathTargetPsd, std::string pathTargetGlobal, std::string solverType, TimeStep* timeStepPtr, double tFinal, std::string pbeMethod, int partPerCluster, int writeFreq);
    
    NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, std::string pathProject, std::string pathTargetPsd, std::string pathTargetGlobal, std::string solverType, TimeStep* timeStepPtr, double tFinal, std::string pbeMethod, int partPerCluster, int writeFreq, std::string sectionalMethod, GrowthPark* growthParkPtr);
    
    NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, std::string pathProject, std::string pathTargetPsd, std::string pathTargetGlobal, std::string solverType, TimeStep* timeStepPtr, double tFinal, std::string pbeMethod, int partPerCluster, int writeFreq, std::string sectionalMethod, GrowthPark* growthParkPtr, int hybridTrigger);
    
    NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, std::string pathProject, std::string pathTargetPsd, std::string pathTargetGlobal, std::string solverType, TimeStep* timeStepPtr, double tFinal, std::string pbeMethod, int partPerCluster, int writeFreq, std::string sectionalMethod, GrowthPark* growthParkPtr, int hybridTrigger, int showPsdFreq);
    
    NpSolver(SootSourceTerm* sourcePtr, AerosolPhase* aeroSolPtr, Psd* psdPtr, int itTotal, double timePerIt, std::string pathProject, std::string pathTargetPsd, std::string pathTargetGlobal, std::string solverType, TimeStep* timeStepPtr, double tFinal, std::string pbeMethod, int partPerCluster, int writeFreq, std::string sectionalMethod, GrowthPark* growthParkPtr, int hybridTrigger, int showPsdFreq, bool subItWrite, double subItWrite_tmin, double subItWrite_tmax);
    
    
    void solve();
    
    void advanceIteration();
    void advanceIterationSectional();   // for sectional
    void advanceIterationStochastic();  // for npBasic or npResidual
    
    
    void advanceGrowth();
    void advanceGrowthStochasticBin(); // apply surf growth on particles with source terms calculated in each bin
    void advanceGrowthStochasticPart(); // apply surf growth on particles with source terms calculated for each particle. For now only for analytical cases. In order to extend to HomogeneousPhase cases, betaCond(part), alpha(part) and surf(part) will have to be calculated instead of betaCond(bin), alpha(bin) and surf(bin)
    void advanceGrowthSectional(); // apply growth on NvResidual in the sections
    
    // hybrid pbeMethod
    void advanceGrowthStochasticBinHybrid();  // in bins where npl < trigger, particles do not grow
    void advanceGrowthStochasticPartHybrid(); // in bins where npl < trigger, particles do not grow. Source term calculated per particle and not per bin
    void advanceGrowthSectionalHybrid();  // in bins where npl < trigger, additional NvResdidual (corresponding to stochastic particles in the bin) is momentaneously added for growthSectionalStep, and then substracted.
    
    std::vector<double> additionalResidualPerBin();  // see above advanceGrowthSectionalHybrid()
    void addResidualUnderTrigger(std::vector<double> additionalResidual);  // see above advanceGrowthSectionalHybrid()
    void substractResidualUnderTrigger(std::vector<double> additionalResidual);  // see above advanceGrowthSectionalHybrid()
    
    
    void advancePBE();    // takes value of nvT from PSD
                          // reallocate
                          // calculate and set nvT(t+dt) in PSD
    
    
    
    double getTime();
    int getIteration();
    double getTimePerIt();
    
    void setItTotal(int itTotal);
    void setSolverType(std::string solverType);
    
    
    // outputs
    void writePsd(); // pathTarget includes target folder path and fileName
    
    void writePsdClusters(); // m not based on fixed grid but on particles location. Input choice: number of stoch part per plot point.
    void writeIntegralFile();
    void writeIntegralLine();
    void consolePrint();
    
    void writePsdHomogeneousGasPhase();
    void writePsdAnalytical();
    
    void writePsdClustersHomogeneousGasPhase(int partPerCluster);
    void writePsdClustersAnalytical(int partPerCluster);
    std::vector<std::vector<double> > getClustersVector(int partPerCluster);
    std::vector<std::vector<double> > getClustersVectorHomogeneous(int partPerCluster);
    
    // for subIt writePsd
    void writePsdSubIt(double t2);
    void writePsdClustersSubIt(double t2);
    
    void writePsdHomogeneousGasPhaseSubIt(double t2);
    void writePsdAnalyticalSubIt(double t2);
    void writePsdClustersAnalyticalSubIt(int partPerCluster, double t2);
    void writePsdClustersHomogeneousGasPhaseSubIt(int partPerCluster, double t2);
    
    void writeIntegralLineHomogeneousGasPhase();
    void writeIntegralFileHomogeneousGasPhase();
    void consolePrintHomogeneousGasPhase();
    void consolePrintAnalytical();
    
    double deltaNvR_from_residualMassCorrection(int bin, std::vector<double> residualMassCorrection);
    
    double getDfromMassNano(double m);
    
protected:
    
    // Attributes
    SootSourceTerm* m_sourcePtr;
    AerosolPhase* m_aeroSolPtr;
    Psd* m_psdPtr;
    GrowthPark* m_growthParkPtr;
    
    int m_itCurrent;
    
    int m_itTotal;
    double m_timePerIt;
    double m_tStepGrowth;
    
    double m_timeSum;
    double m_tFinal;
    
    int m_writeFreq;
    int m_showPsdFreq;
    bool m_subItWrite;  // writing additional writePsd, for growth subIteration within one agglo iteration. Just for a specified time range
    double m_subItWrite_tmin;
    double m_subItWrite_tmax;
    
    TimeStep* m_timeStepPtr;
    
    std::string m_pathProject;
    std::string m_pathTargetPsd;
    std::string m_pathTargetGlobal;
    
    std::string m_solverType; // "HomogeneousGasPhase", "analytical"
    std::string m_pbeMethod; // "npBasic", "npResidual", "sectional", "npBasicPart", "npResidualPart", "hybrid"
    std::string m_sectionalMethod; // "twoPoints", "park" (only for geometricfs grid) (ok for fs =1, uniform?), "2pointGdt", "2pointUpwindmAvg", "2pointUpwindmSup"
    
    int m_partPerCluster;
    int m_hybridTrigger;  // in a bin, if np < trigger and pbeMethod == "hybrid" the corresponding growth is handled through sectional method
    
    // protected methods
    void iterate();
    
    std::vector<double> alphaVectorPerIt(double dotHplusAtPerIt, double dotHperIt);
    double partAggloSourceTot(double t);   // dotAt
    
    void reallocateSectional(std::vector<double> alphaVecPerIt, double dotHplusAtPerIt);
    void reallocateStochastic(std::vector<double> alphaVecPerIt, double dotHplusAtPerIt);
    
    int findBin(double m);
    void calcAndSetTimeStep(double t);
    
    void correctionYA4();
    
    double dotNv2pointUpwind(int i); // not necessary to create a class as this method is more simple.
    
    void reequilibrate_mAvgVar(int i); // problem can arise if mAvgVari is outside of bin i (because of negative NvRi). In this case, there is a need to transfer part from NvRi to neighboring cells conserving mass and number and targetting new mAvgVari = mAvgVarStochi and transfering to mAvgVarRi+1 or mAvgVarRi-1. Then mAvgVar needs to be recalculated for all bins.
    void check_reequilibrate_all_bins();
    
    static const double rhoSoot;
    static const double cmass;
    static const double pi;
    
};

#endif /* NpSolver_hpp */
