//
//  TimeStep.hpp
//  npsSolver
//
//  Created by Alexandre Bouaniche on 22/01/2018.
//  Copyright © 2018 Alexandre Bouaniche. All rights reserved.
//

#ifndef TimeStep_hpp
#define TimeStep_hpp

#include <stdio.h>

#include <vector>
#include <iostream>
#include <fstream>
#include <string>
#include <cmath>
#include <sstream>

#include "SootSourceTerm.hpp"
#include "Psd.hpp"
#include "GrowthTensor.hpp"
#include "GrowthPark.hpp"

class TimeStep
{
public:
    TimeStep(double alpha, SootSourceTerm* sourcePtr, Psd* psdPtr, std::string tStepType, double maxGcoef, std::string growthtStepType, std::string sectionalMethod, GrowthPark* growthParkPtr);
    
    //double aggloTimeStep(double t, partAggloSourceVector via pointer to source);
    double aggloTimeStep(double nvT, double dotH, double dotAt, std::vector<double> dotAiVec, double tsGrowth);
    double aggloTimeStep(double t);
    
    double growthTargetTimeStep(double t);  // targeted time step as a function of maxGcoef
    double growthTargetTsStochastic(double t);
    double growthTargetTsSectional(double t);
    
    int ratioTstepAggloGrowth(double t);  // aggloTimeStep/growthTargetTimeStep  -> rounded up
    double finalGrowthTimeStep(double t);  // finalGrowthTimeStep = aggloTimeStep / ratio
    
    double getAlpha();
    void setAlpha(double alpha);
    std::string tStepType();
    
    
protected:
    
    SootSourceTerm* m_sourcePtr;
    Psd* m_psdPtr;
    GrowthPark* m_growthParkPtr;
    
    std::string m_tStepType;  // "constant", "fixedAlpha", "growth", "alphaAndgSubsteps"
    std::string m_growthtStepType; // "growthSectional", "growthStochastic"
    std::string m_sectionalMethod; // "twoPoints", "park"
    
    double m_alpha;  // chosen "best" AlphaHplusAtperIt
    double m_maxGcoef;   // maximum relative variation in mass by growth of "section centered" particle during 1 time step
    
    double partAggloSourceTot(double t);
    
};


#endif /* TimeStep_hpp */
